   
#include<iostream>
#include<fstream>
using namespace std;




#include"defo.h"
#include"diag.h"
#include"qrdecmp.h"
#include"multiint.h"
#include"zgfiles.h"




double diagtolerance=1e-14;



void orthochangecoord(double A[][MAX_DIM+1],double C[][MAX_DIM+1],
       double T[][MAX_DIM+1],int pdim)
  //   T= C^t A C
{
  for(int i=1; i<=pdim; i++)
    for(int j=1; j<=pdim; j++){
      double t=0;
      for(int k=1;k<=pdim; k++)
        for(int l=1;l<=pdim; l++)
            t+=C[k][i]*A[k][l]*C[l][j];

      T[i][j]=t;
    }

}


double find_dominant_non_diag(double S[][MAX_DIM+1],int *r,int *c,int pdim)
  // in a symmetric matrix <S> - dim(S)=pdim
  // we look for (*r,*c)= (row,col) for which the value of
  // |S[r][c]| - |S[r][r] - S[c][c]|*diagtolerance
  // is maximal
  // a diagonalization is done, when this quantity is less than or equal to 0
{
  *r=1;
  *c=2;
  double wmax=fabs(S[*r][*c]) - fabs(S[*r][*r] - S[*c][*c])*diagtolerance;

  for(int i=1; i<=pdim; i++)
    for(int j=i+1; j<=pdim; j++){
      double t=fabs(S[i][j]) - fabs(S[i][j] - S[j][j])*diagtolerance;
      if (t > wmax){
        wmax=t;
        *r=i;
        *c=j;
      }
    }
  return(wmax);

}



double findMaxNonDiag(double S[][MAX_DIM+1],int *r,int *c,int pdim)
  // in a symmetric matrix <S>   dim(S)=pdim   we  find
  // (*r,*c)=(row,col) such that the value of
  // |S[r][c]|^2 is maximal
  // function returns
  //  \sum_{k\neq j} S[k][j]^2 - diagtolerace \sum_{k} S[k][k]^2
  //   hence if it is negative then the matrix is dominated by diagonal terms
{
  *r=1;
  *c=2;
  double wmax=S[*r][*c]*S[*r][*c];
  double s2nondiag=2*wmax;
  int i;

  for(i=1; i<=pdim; i++)
    for(int j=i+1; j<=pdim; j++){
      double t=S[i][j]*S[i][j];
      s2nondiag+=2*t;
      if (t > wmax){
 wmax=t;
 *r=i;
 *c=j;
      }
    }

  double s2diag=0.0;
  for(i=1;i<=pdim;i++)
    s2diag+=S[i][i]*S[i][i];

  return(s2nondiag - s2diag*diagtolerance);
}



int sym_m_diagonalize(double S[][MAX_DIM+1],
    double D[][MAX_DIM+1],double C[][MAX_DIM+1],int initCoordC,
           int *rotno,int pdim)
  /* diagonalization of a  S'=(S^T + S)/2

    indexing of matrices starts form 1
   on output - return 1 if success
                      0 if fail

    in case of success
          S'=C D C^T;      C^t S' C = D

    C- is orthogonal (nearly - up to computation errors)
    D- is diagonal (nearly - up to some fixed accuracy)

    then
     columns of C - eigenvectors
                D - eigenvalues

   rotno - is a number of rotations needed
  ----------------------

  if on input intCoordC=1
    then we first try to use matrix C (input values) as first rotation
  otherwise we start with identity
  */
{

  int res=1;
  double dist;
  int p,q;
  int i;

  copymatrix(S,D,pdim);


  /* symmetrization */
  for(i=1;i<=pdim;i++)
    for(int j=i+1;j<=pdim;j++){
      double t=0.5*(D[i][j]+D[j][i]);
      D[i][j]=t;
      D[j][i]=t;
    }

  dist= findMaxNonDiag(D,&p,&q,pdim);

  int cinitc=0;


  if(initCoordC){
    double Sw[MAX_DIM+1][MAX_DIM+1];
    orthochangecoord(S,C,Sw,pdim);
    double w2;
    int p2,q2;
    w2= find_dominant_non_diag(D,&p2,&q2,pdim);

    if (w2 < dist){
      cinitc=1;
      dist=w2;
      p=p2;
      q=q2;
      copymatrix(Sw,D,pdim);
    }

  }

  if(!cinitc) identity(C,pdim);
  int done=(dist <= 0.0);
  while(!done){

    double alpha=(D[p][p] - D[q][q])/D[p][q];
    double beta=0.5*(-2*alpha + sqrt(alpha*alpha +4));
    double si=1/sqrt(1+beta*beta);
    double co=beta*si;

    double wpp,wqq,wpq;
    wpp=co*co*D[p][p] + si*si*D[q][q] - 2*si*co*D[p][q];
    wqq=si*si*D[p][p] + co*co*D[q][q] + 2*si*co*D[p][q];
    wpq=co*si*(D[p][p] - D[q][q]) + (co*co - si*si)*D[p][q];

    D[p][p]=wpp;
    D[q][q]=wqq;
    D[p][q]=wpq;
    D[q][p]=wpq;

    double wp,wq;
    // a multiplication from right by a cell  (co, si) (-si,co)
    // i.e. D * cell
    // col[p]:=co * col[p] - si*col[q]
    // col[q]:=si * col[p] + co*col[q]
    for(i=1;i<=pdim; i++){  // i - a row
      if (!((i==p) || (i==q))){
 wp=co*D[i][p] - si * D[i][q];
 wq=si*D[i][p] + co*D[i][q];
 D[i][p]=wp;
 D[i][q]=wq;
 D[p][i]=wp;
 D[q][i]=wq;
      }
    }
    // C* cell
    for(i=1;i<=pdim; i++){  // i - row
      wp=co*C[i][p] - si * C[i][q];
      wq=si*C[i][p] + co*C[i][q];
      C[i][p]=wp;
      C[i][q]=wq;
    }

    (*rotno)++;

    dist= findMaxNonDiag(D,&p,&q,pdim);
    if(!done)
       done=(dist <= 0.0);
  }
  return(res);
}



void swapEigenVect(double D[][MAX_DIM+1],double C[][MAX_DIM+1],int p, int q, int pdim)
  // in data from the diagonalization I swap p-th and q-th eigenvector
  //  and eigenvalue
  // in D I swap both columns and rows
  // in C columns only
{

  // a column swap in C
  int i;
  for(i=1;i<=pdim; i++){
    double t=C[i][p];
    C[i][p]=C[i][q];
    C[i][q]=t;
  }


  // a column swap in  D
  for(i=1;i<=pdim; i++){
    double t=D[i][p];
    D[i][p]=D[i][q];
    D[i][q]=t;
  }

  // a row swap in  D
  for(i=1;i<=pdim; i++){
    double t=D[p][i];
    D[p][i]=D[q][i];
    D[q][i]=t;
  }


}


void sorteigenvalues(double D[][MAX_DIM+1], double C[][MAX_DIM+1],int pdim)
  // input values are matrices from the diagonalizaton
  // D - is almost diagonal
  // C - eignevectors
{
  for(int i=1; i<=pdim-1 ; i++){
    // after i-th loop I should have last <i>-elements ordered correctly
    for(int j=1;j<=pdim -i; j++){
      if(D[j][j] < D[j+1][j+1]){
        swapEigenVect(D,C,j,j+1,pdim);
      }
    }
  }

}


interval VecXAXVec(interval x[],interval m[][MAX_DIM+1],
           interval y[],int pdim)
     /* return   x^T m y */
{
  interval t[MAX_DIM+1];
  multAV(m,y,t,pdim);
  return(scprod(x,t,pdim));
}





