
#include<iostream>
#include<fstream>
#include<cstdlib>
using namespace std;


#include"defo.h"
#include"kserr.h"


#include"rigkserr.h"
#include"multiint.h"

#include"zglstr.h"
#include"zgfiles.h"


#if !defined(M_PI)
#define M_PI  3.14159265358979323846
#endif


bool all_true_in_tab(const bool iso[],int m,int big_m)
// checks if iso[i] for m<i <=big_m
{
  bool ret=true;
  int k=m+1;
  while(ret && (k<=big_m)){
    ret=iso[k];
    k++;
  }
  return(ret);
}


double diff(const interval e1[],const interval e2[],int maxn)
{
  double r=0;
  for(int i=1; i<=maxn; i++){
    r+=0.5*fabs(e1[i].right_b()-e2[i].right_b()) +
          0.5*fabs(e1[i].left_b()-e2[i].left_b()) ;
  }
  return(r);
}


void writeestm(ostream& out,const interval err[],int dim,const char* comment,
   int writeint /*=1 */)
//  writeint - 1- usual interval , 2- center + [-...+..]
{

  out << "-------------------------------------" << endl;
  out << comment << endl;
  for(int k=1;k<=dim;k++){
    out << "k=" << k << "   " ;
    writeinterval(out,err[k],writeint);
    out  << endl;
  }
  out << "-------------------------------------" << endl;
  out.flush();
}

void  writeTailestm(ostream& out,const interval e1[],int m,int big_m,
      interval c,int s,const char* comment, int writeint /*=1*/)
// writeint 1- usual interval , 2- center + [-...+..]
{
  out << "-------------------------------------" << endl;
  out << comment << endl;
  for(int k=m+1;k <= big_m;k++){
    out << "k=" << k << "   b=" ;
    writeinterval(out,e1[k],writeint);
    out << endl;
  }
  out << "Tail : " << c.right_b() << "/k^" << s << endl;
  out << "-------------------------------------" << endl;
  out.flush();
}


void  writeCompTailestm(ostream& out,interval e1[],interval e2[],int m,int big_m,
      interval &c,int s,const bool iso[], bool isotail)
{
   int writeint;
   writeint=1;  // 1- usual interval , 2- center + [-...+..]
   out << "-------------------------------------" << endl;
   for(int k=1;k <= big_m;k++){
     out << "k=" << k << "    err b=";
     writeinterval(out,e1[k],writeint);
     out  << "   after=";
     writeinterval(out,e2[k],writeint);
     if( diameter(e1[k]).right_b() > 1e-30 ){
       out <<"  diam a/b=" << diameter(e2[k]).right_b()/diameter(e1[k]).right_b() << endl;
     }else{
       out << "a/b not computed, diam(b) <1e-30" << endl;
     }
  }
  out << "Tail : " << c.right_b() << "/k^" << s << endl;
  if(all_true_in_tab(iso,m,big_m))
    out << "Isolation for " << m << "<k <=" << big_m << "   ok " << endl;
  else
    out << "Isolation for " << m << "<k <=" << big_m << "   WRONG " << endl;

  if(isotail)
    out << "Isolation for k > " << big_m << "   ok " << endl;
  else
    out << "Isolation for k >" << big_m << "   WRONG " << endl;

  out << "-------------------------------------" << endl;
  out.flush();
}




void updaterrtab(interval inerr[],interval err[],int maxn,interval mu,
                    interval b[])
{
  interval terr;
  for(int n=1; n<=maxn; n++){
    err[n]=inerr[n];
//    if(mu > 1/sqrt(lambda(n))  ){
//      terr=b[n]/(mu*lambda(n) - sqrt(lambda(n)));
    if(mu > power(interval(n),-2)){
      terr=b[n]/(mu*power(interval(n),4) - power(interval(n),2));



      double l=err[n].left_b();
      if(terr.left_b() >l) l=terr.left_b();

      double r=err[n].right_b();
      if(r>terr.right_b()) r=terr.right_b();

      err[n]=interval(l,r);  // err[n]=intesection(err[n],terr)
    }
  }
}






int findbetweensec(double x[],double sec[][MAX_DIM+1],int secnum,int dim)
{
  int found=0;
  int i=1;
  while((i<secnum) && (!found)){
    if((evalsection(sec[i],x,dim) >= 0) && (evalsection(sec[i+1],x,dim)<0))
      found=1;
    else i++;
  }
  if(!found) i=secnum;
  return i;
}

void create13sections(int partition,double sec[][MAX_DIM+1],int dim)
// we create <partition> - radial section in (1,3)-plane going in  clockwise
// direction 1-section is x=0
{
  for(int i=1;i<=partition;i++){
    for(int j=0;j<=dim;j++)
      sec[i][j]=0.0;
  }
  double angle=M_PI/2;
  double angle_step=2*M_PI/partition;
  for(int i=1;i<=partition;i++){
    //  the vector normal to (cos(angle),sin(angle))
    //   is (sin(angle),-cos(angle))
    sec[i][1]=sin(angle);
    sec[i][3]=-cos(angle);
    angle-=angle_step;
  }
}


int ks_rot13_createbounds(const char *traj,double bias,int dim,
   int partition, int *boxnum,interval boxes[][MAX_DIM+1], double expcoef)
// the function creates up to <partition> boxes containing a trajectory
//    the division is based on the partition of angle in (1,3) plane
////////////////
//////
  //  <traj> - name of the file containing a trajectory in 'a'-coordinates
  //        first line contains <dimintraj> - the dimension of the phase space
  //          a line in <traj> has the following structure
  //      t(time) a_1 a_2 ...a_<dimintraj> \newline
  //  <bias> - we include only those points from <traj> for which t > <bias>
  //  <dim> - the dimension of boxes on output
  ////  expcoef - a coefficient (>1) by which all obtained bounds were
  ////      multiplied to create a small margin (for the tail)
  //  OUTPUT:
  //     *boxnum - the number of boxes
  //      boxes[i] - i-th box, the boxes intersect, to facilitate checks
  //               of an inclusion of a set in a box
  //
{

  int ok=0;

  int *boxnonempty;
  boxnonempty=new int[partition+1];
  for(int i=1;i<=partition;i++)
    boxnonempty[i]=0;


  doublematrix1 *sec;
  sec=new doublematrix1[partition+1];

  create13sections(partition,sec,dim);

  cout << "Reading a trajectory "     << endl;

  int dimintraj;
  ifstream tr(traj);
  if(tr==NULL){
    cerr << "can't open the file [" << traj << "]" << endl;
    #if defined(KRAK) && defined(WIN95)
      cout << "press any key " << endl;
      waitBt();
    #endif
    exit(EXIT_FAILURE);
  }

  tr >> dimintraj;
  char trash[240];
  tr.getline(trash,240); // to eat endl;
  if(dimintraj < 1){
    cerr << "the dimension in file = " << dimintraj << "  - something wrong" <<endl;
    #if defined(KRAK) && defined(WIN95)
      cout << "press any key " << endl;
      waitBt();
    #endif
    exit(EXIT_FAILURE);
  }



  int ldim=dim;
  if(ldim > dimintraj) ldim=dimintraj;

  double t=-1;
  double *p1;
  p1=new double[dimintraj+1];

  double *p_dim;
  p_dim=new double[dim+1];

  long pointno=0;
  while ((!tr.eof()) && (t<bias)){
    readp(tr,dimintraj,&t,p1);
    pointno++;
  }



  // an initialization
  if(!tr.eof()){
    ok=1;
    for(int i=1;i<=ldim;i++) p_dim[i]=p1[i];
    for(int i=ldim+1;i<=dim;i++) p_dim[i]=0;

    int secreg=findbetweensec(p_dim,sec,partition,dim);
    boxnonempty[secreg]=1;
    tointerval(p_dim,boxes[secreg],dim);
  }


  //interval pint[MAX_DIM+1];
  interval *pint;
  pint=new interval[dim+1];
  // a regular loop
  int big_pnum=5000;
  while(!tr.eof() && (pointno <big_pnum)){
    pointno++;
    readp(tr,dimintraj,&t,p1);
    for(int i=1;i<=ldim;i++) p_dim[i]=p1[i];
    for(int i=ldim+1;i<=dim;i++) p_dim[i]=0;
    int secreg=findbetweensec(p_dim,sec,partition,dim);


    for(int i=1;i<=ldim;i++)
      pint[i]=interval(p1[i]);
    for(int i=ldim+1;i<=dim;i++)
      pint[i]=0;

    if(!boxnonempty[secreg]){
      boxnonempty[secreg]=1;
      copyvector(pint,boxes[secreg],dim);
    }else{
      MultiIhull(pint,boxes[secreg],boxes[secreg],dim);
    }
  }
  delete [] pint;
  pint=NULL;
  delete [] p_dim;
  p_dim=NULL;
  delete [] p1;
  p1=NULL;
  delete [] sec;
  sec=NULL;

  cout << pointno << " points read " << endl;

 // gluing together consecutive boxes
//  interval box1[MAX_DIM+1];
  interval *box1;
  box1=new interval[dim+1];
  int box1nonempty=boxnonempty[1];
  if(boxnonempty[1])
    copyvector(boxes[1],box1,dim); // to have a copy of first box
                   //  it has to be glued with the last one
  for(int i=1;i<partition;i++){
    int next=i+1;
    if(boxnonempty[i] && boxnonempty[next]){
      MultiIhull(boxes[i],boxes[i+1],boxes[i],dim);
    }
    if(boxnonempty[i] && (!boxnonempty[next])){
       // we don't do anything
    }
    if((!boxnonempty[i]) && !(boxnonempty[next])){
       // we don't do anything
    }
    if(!(boxnonempty[i])  && boxnonempty[next]){
      copyvector(boxes[next],boxes[i],dim);
      boxnonempty[i]=1;
    }
  }
  // the last box is treated separately because it has to glued with box1
  if(boxnonempty[partition] && box1nonempty){
      MultiIhull(boxes[partition],box1,boxes[partition],dim);
  }
  if(boxnonempty[partition] && !box1nonempty){
       // we don't do anything
  }
  if(!(boxnonempty[partition]) && !box1nonempty){
       // we don't do anything
  }
  if(!(boxnonempty[partition])  && box1nonempty){
    copyvector(box1,boxes[partition],dim);
    boxnonempty[partition]=1;
  }
  delete [] box1;
  box1=NULL;

  // dropping empty boxes
  *boxnum=0;
  for(int i=1;i<=partition;i++){
    if(boxnonempty[i]){
      *boxnum+=1;
      if(*boxnum != i)
        copyvector(boxes[i],boxes[*boxnum],dim);
    }
  }



  // an extension of each box by a factor <expcoef>
  if(expcoef > 1){
    // interval r[MAX_DIM+1];
    interval *r;
    r=new interval[dim+1];
    for(int i=1;i<=*boxnum;i++){
      DecomposeIMulti(boxes[i],r,dim);
      for(int j=1;j<=dim;j++)
         r[j]*=expcoef;

      for(int j=1;j<=dim;j++)
         boxes[i][j]+=r[j];

    }
    delete [] r;
    r=NULL;
  }


  delete [] boxnonempty;
  return ok;
}









