#ifndef _metnum_h_
#define _metnum_h_
#include"defo.h"
#include"zdynsys.h"



void remove_taylor_arrays();
// this function frees the memory allocated for some global arrays used by
//  taylor and dertaylor


int taylor(Ode &ode,int order, interval h, interval x[],interval y[],
        int calcrem, interval rem[]);
// we assume that derivatives of the vector field higher than 3 order vanish
 // order - order of the numerical method
 // h - time step
 // x - point at which moves under flow
 // y - its image after time h (computed only when !calcrem)
 //  rem - remainder from Taylor formula (before multiplication by h^{order +1}
 //       to have this computed correctly we need to use as <x> a rough enclosure.
  //        <rem> is  computed only when <calcrem>=1

 //
 //   to move forward by time step <h> we call taylor
 //   twice : 1) for x and calcrem=0
 //           2) for x=enclosure and calcrem=1

int taylor(Ode &ode,int order, double h, double x[],double y[],
        int calcrem, double rem[]);

void rk4(Ode &ode,interval h, interval x[],interval y[]);



void rk4(Ode &ode, double h, double x[],double y[]);


int dertaylor(Ode &ode,int order, double h, double x[],
       double Vencl[][MAX_DIM+1],double y[], double Dx[][MAX_DIM+1],
       int calcrem, double DxRem[][MAX_DIM+1]);


int dertaylor(Ode &ode,int order, interval h, interval x[],
       interval Vencl[][MAX_DIM+1],interval y[],
       interval Dx[][MAX_DIM+1],int calcrem, interval DxRem[][MAX_DIM+1]);
 // order - an order of the numerical method
 // h - a time step
 // x - an interval vector on which we calculate the derivative of Taylor method
 //      when <calcrem>=0 - then this is our value of [x_k], otherwise
 //      it is [W_1] - enclosure for \varphi([0,h],[x_k])
 // Vencl - an interval matrix , used only when <calcrem> -
 //          this an enclosure of \frac{\partial \varphi}{\partial x}([0,h],x)=
 //               V([0,h],x); this a matrix W3 in 'C1-Lohner algorithm'
 //        otherwise is ignored (which is equivalent to taking Vencl=Id)
 //  calcrem - 1 - we compute remainder term - dertalor of order <order+1>
// ON OUTPUT:
 // y - an enclosure for  TaylorExpansion(h,x) up to <order>
 // Dx - a derivative of the Taylor method:  : D TaylorExpansion(h,x) up to <order>
 // DxRem - the remainder term, before a multiplication by h^{order+1}:
  //  d^{order}/dt^{order}( \frac{\partial f}{\partial x} V)([x],id)*Vecnl/(order+1)!
 //  y,Dx are computed only when <!calcrem>
 //  DxRem is computed only when <calcrem>

 // we suppose that higher derivates (>2) of vector field vanish

//  for C^0-Lohner - calcrem=0
//  to use this function in C1-Lohner we need to call this function twice
//   1) - for x (our starting set) calcrem=0 - we compute Taylor expansion
//               for variational matrix
//   2) for x=enclosure of \varphi([0,h],x),
//    Vencl=W3=enclosure of
//         \frac{\partial \varphi}{\partial x}([0,h],x)=V([0,h],x)
//          and calcrem=1


void OneStepVar(Ode &ode,double h,double x[], double Dx[][MAX_DIM+1],
     double Dxn[][MAX_DIM+1], int dimVar,int order);
// performs one step for Jacobian matrix of the flow
//  using Taylor method
// INPUT values:
//  x - current position
//  Dx - current value of Jacobian matrix
//  dimVar - we will do Galerkin proj. onto first <dimVar> -variables
// OUTPUT VALUE:
//  Dxn - value of Jacobian matrix after time +<h>


#endif
