#include<iostream>
#include<fstream>
using namespace std;


#include"defo.h"
#include"func.h"
#include"faq.h"
#include"zgfiles.h"
#include"qrdecmp.h"
#include"perfixp.h"
#include"section.h"

#include"multiint.h"
#include"zdynsys.h"
#include"rigpoinc.h"


const  int meth_newtonsearch=1;
const  int meth_forwarditeration=0;

char *meth_fixed_point_name(int method,char* name,int maxlen)
{
  if(method==meth_newtonsearch)
    strncpy(name,"Newton search",maxlen-1);
  if(method==meth_forwarditeration)
    strncpy(name,"Search by forward iteration",maxlen-1);

  return name;
}


int odefindfixp(Ode &ode,double y[],double h,int order, double tmax,double err)
// function moves forward <x> up to a time <tmax> in the hope that the
//  solution will converge to a fixed point;
//<h> - the time step
// <order> - the order of the method
//  <err> -is a tolerance parameter
//   point is considered to be a fixed point if meets two criteria
//    |ode.vfield(x)| < err
//     <x> does not move more then <err> in time 0.05*<tmax>
//  if order <=4 then 4-th order Runge-Kutta is used
//               otherwise we use taylor method
{

  int ret=0;
  h=ode.timestep(h);

  double xn[MAX_DIM+1];
  double t=0;
  int infixp=0;
  double tinfixp=0;
  double vfield[MAX_DIM+1];
  double fixp[MAX_DIM+1],x[MAX_DIM+1];

  copyvector(y,x,ode.dim);

  while((!ret) && (t<tmax)){
    for(int i=1;i<=ode.dim;i++)
      vfield[i]=ode.vfield(i,x);
    if(order <= 4){
      rk4(ode,h,x,xn);
    } else{
      taylor(ode,order,h,x,xn,0,NULL);
    }
    ode.adjustenergy(xn);
    t+=h;
    if(infixp){
      if((normMax(vfield,ode.dim) < err) &&
           (distNormMax(fixp,xn,ode.dim) < err)){
        tinfixp+=h;
      } else {
        infixp=0;
        tinfixp=0.0;
      }
    } else {
      if((normMax(vfield,ode.dim) < err) &&
           (distNormMax(x,xn,ode.dim) < err)){
        infixp=1;
        tinfixp=h;
        for(int i=1;i<=ode.dim;i++)
          fixp[i]=x[i];
      }
    }
    if(infixp){
      ret=((tinfixp >= 50*h) || ( tinfixp > 0.01*tmax));
    }
    for(int i=1;i<=ode.dim;i++)
      x[i]=xn[i];
  }
  if(ret) copyvector(x,y,ode.dim);
  return(ret);

}



int fixpdfanal(Ode &ode,double fixp[],int *no_unstabledir,double *stabfactor)
// <fixp> is a fixed point for <ode>
// On output *no_unstabledir - is the number of unstable directions
//    for linearization at fixp
//      *stabfactor - (for flow) max reale part of eigenvalue of df
// returns 1 - when all went ok
//         0 - otherwise
{
  double df[MAX_DIM+1][MAX_DIM+1];

  for(int i=1;i<=ode.dim;i++)
    for(int j=1;j<=ode.dim;j++)
      df[i][j]=ode.dvfield(i,j,fixp);

  return(calcnounstable(df,ode.dim,1,no_unstabledir,stabfactor));

}



int newtonfixp(Ode &ode,double x[],double a[MAX_DIM+1][MAX_DIM+1],int maxiterno,
               double err)
  // function looks for zero of the vector field <ode>
  // using Newton-type algorithm
  //  x->N(x) = x - df(x)^{-1}f(x), with df(x)^{-1} replaced by <a>
  // for this algorithm to work <x> - our initial guess must be close to true
  // zero
  //  a - shall be close to df(x)^{-1}
  //  ---------
// on output x - contains the fixed point found
{
  int itno=1;
  int found=0;
  double xn[MAX_DIM+1],xn1[MAX_DIM+1];
  for(int i=1;i<=ode.dim;i++)
    xn[i]=x[i];
  while(!found && (itno <=maxiterno)){
    double vfield[MAX_DIM+1];
    for(int i=1;i<=ode.dim;i++)
      vfield[i]=ode.vfield(i,xn);

    for(int i=1;i<=ode.dim;i++){
      double t=0;
      for(int j=1;j<=ode.dim;j++)
        t+=a[i][j]*vfield[j];
      xn1[i]=t;
    }
    // at this moment xn1=a*xn;

    for(int i=1;i<=ode.dim;i++)
      xn1[i]=xn[i]-xn1[i];


    if((normMax(vfield,ode.dim) < err) &&
                 (distNormMax(xn,xn1,ode.dim) < err)){
      found=1;
    }
    for(int i=1;i<=ode.dim;i++)
      xn[i]=xn1[i];
    if(!found){
      itno++;
    }

  }
  if(found){
    for(int i=1;i<=ode.dim;i++)
      x[i]=xn[i];
  }
  return(found);
}







int newtonfixp(Ode &ode,double x[],int maxiterno,double err)
  // function looks for zero of the vector field <ode>
  // using Newton-type algorithm
  //  x->N(x) = x - df(x)^{-1}f(x), with df(x)^{-1} replaced by <a>
  // for this algorithm to work <x> - our initial guess must be close to true
  // zero
  //  ---------
// on output x - contains the fixed point found
{
  double a[MAX_DIM+1][MAX_DIM+1];
  double df[MAX_DIM+1][MAX_DIM+1];
  for(int i=1;i<=ode.dim;i++)
    for(int j=1;j<=ode.dim;j++)
      df[i][j]=ode.dvfield(i,j,x);



  if(qrinverse(df,a,ode.dim))
    return newtonfixp(ode,x,a,maxiterno,err);
  else return 0;
}


int newton0_1dim(double (*f)(double),double (*df)(double),double xstart,double *x,
    double precision,int maxiter /*=100*/)
// newton algorithm for 1-dim problem f(x)=0
//  x-> x - df^{-1}*f(x)
// xstart - starting value for iteration
// *x - solution of f(x)=0 on output
// precison - if |f(x)| < precison then we consider assume that this x is ok
// maxiter - maximum numeber of iterates
{
  *x=xstart;
  int iter=0;
  int found=0;
  while((!found) && (iter<maxiter)){
    double fx=f(*x);
    if(fabs(fx)>precision){
      double dfx=df(*x);
      *x=*x - fx/dfx;
    } else found=1;
    iter++;
  }
  return(found);
}





int newtonsearch(Map &map,int exp,int fixp,double x[],int maxiterno,
    double maxstep,double err,int *iter)
  // function looks for fixed point (or zero) of <map>^<exp>
  // using Newton-type algorithm
  // if <fixp> then we look for fixed point
  //   otherwise we look for zero
  //
  //  x->N(x) = x - df(x)^{-1}f(x)
  // for this algorithm to work <x> - our initial guess must be close
  //  <maxstep> - we don't allow bigger step than this number
  // <err> - when we are this close to fixed point (or zero)(in Euclidean norm),
  //   then we   stop
  //  ---------
// on output x - contains the fixed point (or zero) found
//  *iter - is equal to the number of iterations performed
{
  int itno=1;
  int found=0;
  int ok=1;
  double xn[MAX_DIM+1];
  copyvector(x,xn,map.dim);
  while(!found && (itno <=maxiterno)  && ok){
    double fx[MAX_DIM+1];
    double df[MAX_DIM+1][MAX_DIM+1];
    ok=map.iterate(exp,xn,fx,df);
    if(!ok) continue;


    if(fixp){  // for fixed point we look for zero of the map
               //  Id - f
      for(int i=1;i<=map.dim;i++)
        fx[i]= xn[i] - fx[i];
     // now fx=x - f(x)

      // a computation of df=Id - df
      for(int i=1;i<=map.dim;i++)
        for(int j=1;j<=map.dim;j++){
          df[i][j]=-df[i][j];
          if(i==j) df[i][j]+=1.0;
        }
    }

    found=(normEucl(fx,map.dim)<err);
    if(found) continue;

    double invdf[MAX_DIM+1][MAX_DIM+1];
    double d[MAX_DIM+1];
    if(map.dim<=2)
      ok=inverse(df,invdf,map.dim);
    else
      ok=qrinverse(df,invdf,map.dim);
    if(!ok) continue;
    multAV(invdf,fx,d,map.dim);  // d=df^{-1}(x)f(x)

    double n=normEucl(d,map.dim);
    if(n > maxstep){  // we make d shorter
      for(int i=1;i<=map.dim;i++)
        d[i]*=maxstep/n;
    }

    for(int i=1;i<=map.dim;i++)
      xn[i]-=d[i];

    if(!found) itno++;

  }
  if(found){
    *iter=itno;
    for(int i=1;i<=map.dim;i++)
      x[i]=xn[i];
  }
  return(found);
}



int test_attracting(Map &map,int iterno,double x[],int sendwarning)
// this function checks, if <x> a candidate for the fixed point of <map^iterno>
//  is attracting, by looking at the diagonalization of df(x)
{
  double df[MAX_DIM+1][MAX_DIM+1];
  double xn[MAX_DIM+1];
  double eigenval[MAX_DIM+1][3];
  int maxiterno=1000;
  double bias=1e-6;  // previously 1e-12
  double relbias=1e-4;

  double p[MAX_DIM+1][MAX_DIM+1];
  double pinv[MAX_DIM+1][MAX_DIM+1];

  int normalize=1;

  int attr=1;

  int ret=map.iterate(iterno,x,xn,df);
  if(ret){
    double diff[MAX_DIM+1];
    for(int i=1;i<=map.dim;i++)
      diff[i]=x[i]-xn[i];
    double normx=normMax(x,map.dim);
    if(normx > 0){
      double normdiff=normMax(diff,map.dim);
      if((normdiff > 1e-2*normx) && sendwarning){
        cout << " norm of x = " << normx << endl;
        cout << "Warning: relatively large x - f(x) in test of attracting f.p. " << normdiff << endl;
      }
    }
  }

  if(ret){

    ret=qrdiagonalize(df,p,pinv,eigenval,map.dim,maxiterno,bias,relbias,
                        normalize);
    if(ret){

      int cno=1;
      while(cno<=map.dim){
        if(normEucl(eigenval[cno],2) > 1) attr=0;
        cno++;
      }
    }
  }
  return(ret && attr);
}



int findattrfpMap(Map &map,double xs[],int itno,int max_iter, double err,int *iterfound)
// the function searches for an attracting fixed (periodic) point for <map>
//  by a naive iteration , hoping for convergence to fixed point
//   xs - on input - the starting point, on output - the fixed point for
//       <map^itno>
//   max_iter - upper limit for the number of iterates of <map>
//   err - the tolerance (error) in the criterion  for fixed point
//   if the point x is found such that |map(x) - x|_{max} < err, then 1 is returned
//                  xs - contains the fixed point found
//                  *iterfound - the number of iterates
///  if we don't succed then 0 is returned and xs stays unchanged
/////////////////
{
  int found=0;
  *iterfound=0;

  double x[MAX_DIM+1],xn[MAX_DIM+1];
  copyvector(xs,x,map.dim);

  int ok=1;
  int iter=1;
  while((iter<=max_iter) && (!found) && ok){
    ok=map.iterate(itno,x,xn);

    if(ok){
      found=(distNormMax(x,xn,map.dim) < err);
      for(int i=1;i<=map.dim; i++)
        x[i]=xn[i];
    }
    iter++;
  }
  if(found){
    copyvector(x,xs,map.dim);
    *iterfound=iter-1;
  }
  return(found);
}



int improvefixpoint(Map &map,int iterno,double x[],int max_iter,double err)
// On input: x is a candidate for a fixed point for map^iterno
// On output - if everything went ok (1 is returned) x - is better estimate
//           for fixed point of map^{iterno}
////////////
//we first check whether the candidate for the fixed (periodic) point  is attracting -
//        if this is the case then we just iterate forward
// otherwise use the Newton method
{
  int ret;

  // I need to test if this is an attracting fixed point
  if(test_attracting(map,iterno,x,1)){ // we compute df and look at eigenvalues
    int iterused;
    ret=findattrfpMap(map,x,iterno,max_iter,err,&iterused);

    if(!ret){
      cerr << "Fixed point not found" << endl;
      #if defined(WIN95) && defined(KRAK)
         cerr << "press any key " << endl;
         waitBt();
      #endif
    }
  } else {  // by nonrigorous Newton method
    int fixp=1;
    double maxstep=0.1;
    int iter;
    ret=newtonsearch(map,iterno,fixp,x,max_iter,maxstep,err,&iter);
  }
  return ret;
}

void   writePMaptraj(ostream &report,FPoincareMap *PMap,int mapiter,double xb[])
// writes : Pmap^i(xb) for i=0,,,,mapiter-1
//   xb - is given in the section  coordinates
{
  double x1[MAX_DIM+1],x2[MAX_DIM+1];
  double xf[MAX_DIM+1];
  report << endl << "The orbit in ambient space coordinates: " << endl;
  PMap->pInsection->sectiontofull(xb,xf);
  saveAllCoord(report,xf,PMap->pInsection->dim);
  copyvector(xb,x1,PMap->dim);
  for(int i=1;i<=mapiter;i++){
    PMap->map(x1,x2);
    PMap->pInsection->sectiontofull(x2,xf);
    saveAllCoord(report,xf,PMap->pInsection->dim);
    copyvector(x2,x1,PMap->dim);
  }
  report << endl << endl;
}



