#include<iostream>
#include<fstream>
#include<iomanip>
using namespace std;

#include<cstdlib>

#include"defo.h"
#include"func.h"
#include"multiint.h"

#include"zgfiles.h"


int fileexists(const char *filename)
{
  return(ifstream(filename)!=0);
}


double normMaxT(const double ptslist[][MAX_DIM+1],int pdim,int num_of_pts)
{
  double ret=0;
  for(int i=1;i<=num_of_pts;i++){
    double t=normMax(ptslist[i],pdim);
    if(t>ret) ret=t;
  }
  return(ret);
}


void delptsfromtable(double ptslist[][MAX_DIM+1],int *num_of_pts,int pts_dim,
     int num_pts_to_del)
// from the table <ptslist> containing <*num_of_pts> points of dimension <pts_dim>
//  we remove first <num_pts_to_del> points;
//  On output also we update <*num_of_pts>
{
  if(*num_of_pts > num_pts_to_del){
    if (num_pts_to_del > 0){
      for(int i=num_pts_to_del+1; i<=*num_of_pts; i++)
        for(int j=1;j<=pts_dim;j++)
          ptslist[i-num_pts_to_del][j]=ptslist[i][j];
    }
    *num_of_pts-=num_pts_to_del;
  } else{
    *num_of_pts=0;
  }
}



int readptstable(const char name[],double points[][MAX_DIM+1],int *num_of_pts,
              int pdim,int max_num)
// pdim - is the dimension of <points>, depending on dim_in_file
//                    we truncate points from <name> dim_in_file > pdim
//           or set points[i][pdim >= j > dim_in_file]
// max_num - is the maximum number of points
// it is assumed the the first line in file <name> contains the dimension (dim_in_file)
//  and in the following lines we have coordinates of  points separated by
//       one or more spaces
{

  ifstream fhandle(name);
  if(!fhandle){
    cerr <<  "File not found : " << name << "\n";
    #if defined(WIN95) && defined(KRAK)
      cerr << "press any key " << endl;
      waitBt();
    #endif
    return(0);
  }

  int diminfile;
  fhandle >> diminfile;
  char trash[240];
  fhandle.getline(trash,240); // to eat endl;

  int dimread=diminfile;
  if(dimread > pdim) dimread=pdim;


  *num_of_pts=0;

  while ((!fhandle.eof()) && ((*num_of_pts) < max_num)){
    (*num_of_pts)++;
    for(int i=1;i<=dimread;i++)   fhandle >> points[*num_of_pts][i];
    for(int i=dimread+1;i<=pdim;i++) points[*num_of_pts][i]=0;
    fhandle.getline(trash,240); // to eat endl;
  }
  (*num_of_pts)-- ;  // because allways 0 is read at the end WHY???
  return(1);
}


void save_table(ostream& report ,const double ptslist[][MAX_DIM+1],int num_of_pts,
                  int diminfile)
// we save ptslist[1],...ptslist[num_of_pts]
{


  #if defined(BORLAND)
  long int
  #else
  ios::fmtflags
  #endif
  oldflags=report.flags();


  report.setf(ios::scientific);
  int point_no;
  int i;
  for (point_no=1; point_no <= num_of_pts; point_no++){
    if (point_no > 1) report << endl;

    for(i=1;i<=diminfile;i++)
      report << setw(8) << ptslist[point_no][i] << "  ";
  }
  report.flags(oldflags);
}



void save_table(const char name[],const double ptslist[][MAX_DIM+1],int num_of_pts,
                  int diminfile)
{
  ofstream fhandle(name);

  save_table(fhandle,ptslist,num_of_pts,diminfile);

}




void swap_vect(double vect[][MAX_DIM+1],int pts_dim,int i1,int i2)
{
  double temp[MAX_DIM+1];
  for(int i=1;i<=pts_dim; i++)
    temp[i]=vect[i1][i];
  for(int i=1;i<=pts_dim; i++){
    vect[i1][i]=vect[i2][i];
    vect[i2][i]=temp[i];
  }
}


void sortTab(double points[][MAX_DIM+1],int num_of_pts,int pts_dim,const Func &func)
{
  for(int i=1;i<=num_of_pts;i++){
    for(int j=1;j<=num_of_pts -i;j++){
      if (func.eval(points[j]) > func.eval(points[j+1]))
        swap_vect(points,pts_dim,j,j+1);
    }
  }

}


int  find_pt_inTab(double current_r,const double points[][MAX_DIM+1],int pts_number,
       int pts_dim,const Func &func,double x[],int pdim)
//   in a table <points>  coordinates of <pts_dim>-dimensional vectors
  // we assume that the table <points> is ordered increasingly according
  //    to the value of <func> i.e.  func(points[i]) < func(points[i+1])
  // using table <points> are representing a set parameterized by the
  //  value of func(x) we try solve the equation:
  //    func(x)=current_r,   dim(x)=pdim (we suitabley truncate/expand <points[i]>
  //  1 is returned if success 0 otherwise
{
  for(int i=1;i<=pdim; i++)
    x[i]=0.0;
  double rdn,rup,rt;
  int pdn,pup,t;
  rdn=func.eval(points[1]);

  rup=func.eval(points[pts_number]);


  int found=0;
  if (current_r <= rdn){
    found=1;
    pdn=1;
    pup=1;
  }
  if (current_r >= rup){
    found=1;

    pdn=pts_number;
    pup=pts_number;
  }
  if (!found){
    pdn=1;
    pup=pts_number;
  }
  while ((!found) && (pup - pdn > 1)){
    t=(pup + pdn)/2;

    rt=func.eval(points[t]);

    if (current_r==rt){
      found=1;
      pup=t;
      pdn=t;
      rup=rt;
      rdn=rt;
    } else{
      if (current_r < rt){
        pup=t;
        rup=rt;
      } else {
        pdn=t;
        rdn=rt;
      }
    }
  }
  if (!found) found=(pup == pdn+1);

  if (found){
    if (pdn==pup){
      int i=1;
      while ((i<=pdim) && (i<=pts_dim)){
        x[i]=points[pdn][i];
        i++;
      }
    } else{
      double coeff;
      coeff=(current_r - rup)/(rdn-rup);
      int i=1;
      while ((i<=pdim) && (i<=pts_dim)){
        x[i]=coeff*points[pdn][i] + (1-coeff)*points[pdn+1][i];
        i++;
      }
    }
  }
  return(pdn);
}

void spec_copyvect(const double zr[],double y[],int zdim,int ydim)
// dim(zr)=  zdim
// dim(y) = ydim
// On output y=zr
{
  for(int i=1; i<=ydim; i++)
    y[i]=0.0;
  for(int i=1; ((i<=ydim) && (i<=zdim)); i++)
    y[i]=zr[i];
}

int  findClosePtinTab(const double x[],const double points[][MAX_DIM+1],int pts_number,
     int pts_dim, double (*normfunc)(const double[],int),int xdim)
     // dim(x)=xdim
     // dim(points[i])=pts_dim
     // on output we have an index of a point in <points>, which is the closest
     // point to <x>
{
  int imin;
  double min_dist;
  double diff[MAX_DIM+1];
  double test;

  double y[MAX_DIM+1];

  int ldim;
  ldim=(xdim <= pts_dim)?xdim : pts_dim;

  for(int point_no=1; point_no<=pts_number; point_no++){
    spec_copyvect(points[point_no],y,pts_dim,ldim);
    for(int i=1; i<=ldim; i++)
      diff[i]=x[i]-y[i];
    if (point_no==1){
      imin=1;
      min_dist=(*normfunc)(diff,ldim);
    } else
    {
      test=(*normfunc)(diff,ldim);
      if (test < min_dist){
        imin=point_no;
        min_dist=test;
      }
    }
  }
  return (imin);
}






void readfirstline(const char name[],char line[])
{
  ifstream  fhandle(name);
  fhandle.getline(line,40);
}



int readArr2D(const char name[],double tab[][MAX_DIM+1],int *diminfile)
  // reads 2D array from the text file
  // 1st line have to begin with the dimension - that a comment might be put
  // from the second line on we have the array entries
{

  ifstream fhandle(name);
  if(!fhandle){
    cerr << "File not found : " << name << "\n";
    #if defined(WIN95) && defined(KRAK)
      cerr << "press any key " << endl;
      waitBt();
    #endif
    return(0);
  }

  fhandle >> *diminfile;
  char trash[240];
  fhandle.getline(trash,240); // to eat endl - and the eventual comment following
                              // the dimension


  for(int i=1;i<=*diminfile;i++)
    for(int j=1; j<=*diminfile; j++)
      fhandle >> tab[i][j];

  return(1);

}



int readVect(const char name[],double x[],int *diminfile)
{

  ifstream fhandle(name);
  if(!fhandle){
    cout << "File not found : " << name << "\n";
    return(0);
  }

  fhandle >> *diminfile;
  char trash[240];
  fhandle.getline(trash,240); // to eat endl - and the eventual comment following
                              // the dimension



  for(int i=1;i<=*diminfile;i++)
    fhandle >> x[i];

  return(1);

}


int  copypoints(ostream & dest,const char src[],int dim,int bias)
{
  int retval=1;
  ifstream src_file(src);
  if(!src_file) {
    cerr << "Procedure  copypoints - can't open file [" << src <<  "]" <<
      "  for reading " << endl;
       #if defined(KRAK) && defined(WIN95)
                              waitBt();
     #endif
    exit(1);
  }
  int diminfile;
  src_file >> diminfile;
  char trash[240];
  src_file.getline(trash,240); // to eat endl - and the eventual comment following
                              // the dimension


  int pointno=1;
  double *x,*y;
  x=new double[diminfile+1];
  y=new double[dim+1];
  while(!src_file.eof()){
    for(int i=1;i<=diminfile;i++) src_file >> x[i];
    if(pointno > bias){
      zero(y,dim);
      for(int i=1;(i<=dim) && (i<=diminfile); i++)
        y[i]=x[i];

      for(int i=1;i<=dim;i++)
         dest << setw(8) << y[i] << "  ";
      dest << endl;
    }
    pointno++;
  }

  delete [] y;
  delete [] x;
  return(retval);
}

int addfilespoints(const char dest[],const char name1[],const char name2[],
               int dim,int bias)
  // function writes to file <dest> points from files <name1>, <name2>
  //   <dim> is the dimension of points (may be different then the one
  //                      in files)
  //   <bias> - number of points on the beginning of both files not included
  //        into <dest>
{
  int retval;

  ofstream fhandle(dest);
  retval=(fhandle!=NULL);
  if(!retval){
    cerr << "Procedure addfilespoints - can't open file [" << dest <<  "]" <<
      "  for writing " << endl;
       #if defined(KRAK) && defined(WIN95)
                              waitBt();
     #endif
    exit(1);
  }
  fhandle << dim << "  - the dimension " << endl;
  if(retval)
    retval=copypoints(fhandle,name1,dim,bias);
  if(retval)
    retval=copypoints(fhandle,name2,dim,bias);

  return(retval);
}



void writediam(ostream& report,const interval xn[],int pdim)
// writes to <report> diameters of each coordinate of <pdim>-dimensional vector
{
  report << "diameters:" << endl;

 #if defined(BORLAND)
  long int
 #else
    ios::fmtflags
 #endif
 oldflags=report.flags();

  report.setf(ios::scientific);

  for(int i=1;i<=pdim;i++){
    report << diameter(xn[i]).right_b() << "  ";
  }
  report << endl;

  report.flush();
  report.flags(oldflags);
}


void writeinterval(ostream &out,const interval & r, int howtowrite)
//  howtowrite=1 - standard interval
//            =2 - center + [-d,d]
{
  if(howtowrite==1){
    out << r << "    ";
  }
  if(howtowrite==2){
    double diam;
    interval midp=r;
    iSplit(&midp,&diam);
    interval d=interval(-diam,diam);
    out << midp.left_b() << " +" << d.right_b() << "*[-1,1]   ";
  }
}




void saveAllCoord(ostream& report,const interval x[],int pdim,int howtowrite/*=0*/,int
    write_ind /*=0 */,int breakline /*= 0*/)
// howtowrite == 0 - only the mid-point is written
//                 1 - interval, standard form
//                 2 - interval : center + [-d,d]
//                 3 - in a program format 'x[i]=interval(....,...)
//    if howtowrite ==3 then the other parameters are ignored
// write_ind = 1 - then write an index in parentheses
// breakline !=0 , then we pass to a new row if an index % breakline ==0
{


 #if defined(BORLAND)
  long int
  #else
    ios::fmtflags
  #endif
  oldflags=report.flags();


  report.setf(ios::scientific);
  for(int i=1;i<=pdim;i++){
    if(howtowrite==3){
      report << "x[" << i << "]=interval(";
      report << x[i].left_b() << ", " << x[i].right_b() ;
      report << ");" << endl;
    } else{
      if(write_ind) report << "(" << i << ")";
      if(howtowrite==0) report << mid(x[i]).right_b() << "  ";
      else writeinterval(report,x[i],howtowrite);
       if(breakline){
        if((i % breakline==0) && (i!=pdim)) report << endl;
      }
    }
  }
  report << endl;
  report.flush();
  report.flags(oldflags);

}


void saveAllCoord(ostream& report,const double xn[],int pdim,int  ,int write_ind /*=0 */,
   int breakline /*= 0*/)
{

  for(int i=1;i<=pdim;i++){
    if(write_ind) report << "(" << i << ")";
    report << xn[i] << "  ";
    if(breakline){
      if((i % breakline==0) && (i!=pdim)) report << endl;
    }
  }
  report << endl;
  report.flush();
}


void writeDiff(ostream &report,const double x1[],const double x2[],int pdim,
      int write_ind /*=0 */, int breakline /*= 0*/)
// writes to the report:   x1-x2
{
  double *d;
  d=new double[pdim+1];
  for(int i=1;i<=pdim;i++)
    d[i]=x1[i]-x2[i];

  saveAllCoord(report,d,pdim,0,write_ind,breakline);

  delete [] d;
}


void saveAllCoord(const char* report,const double x[],int pdim)
{
  ofstream fhandle(report,ios::out);
  fhandle << pdim << endl;
  saveAllCoord(fhandle,x,pdim);

}




void readvector(istream& report,double xn[],int pdim)
{
 for(int i=1;i<=pdim;i++)
    report >> xn[i];
}

void readvector(istream& report,interval xn[],int pdim)
{
 for(int i=1;i<=pdim;i++)
    report >> xn[i];
}


void savesection(ostream& report,const double sec[],int pdim)
{

  for(int i=0;i<=pdim;i++)
    report << sec[i] << "  ";
  report << endl;
  report.flush();
}


void savesection(ostream& report,const interval sec[],int pdim)
{
  for(int i=0;i<=pdim;i++)
    report << sec[i] << "  ";
  report << endl;
  report.flush();
}


void readsection(istream& report,interval sec[],int pdim)
{
  for(int i=0;i<=pdim;i++)
    report >> sec[i];
}

void readsection(istream& report,double sec[],int pdim)
{
  for(int i=0;i<=pdim;i++)
    report >> sec[i];
}


void writediam(ostream& report,const interval Dx[][MAX_DIM+1],int pdim)
// writes to <report> diameters of each entry of <pdim>^2-dimensional table
{
#if defined(BORLAND)
  long int
 #else
    ios::fmtflags
 #endif
 oldflags=report.flags();


  report << "diameters: " << endl;
  report << "dim="<< pdim << endl;

  report.setf(ios::scientific);
  for(int i=1;i<=pdim;i++){
    for(int j=1;j<=pdim;j++)
      report << diameter(Dx[i][j]).right_b() << " ";
    report << endl;
  }
  report.flags(oldflags);
}



void saveArray(ostream& report,const interval Dx[][MAX_DIM+1],int pdim,
   int howtowrite /* =1*/,bool write_ij /*= false*/)
// howtowrite == 0 - only the mid-point is written
//                 1 - interval, standard form
//                 2 - interval : center + [-d,d]
// if write_ij , then before each term we write the row and column in parentheses
{
  #if defined(BORLAND)
  long int
 #else
    ios::fmtflags
 #endif
 oldflags=report.flags();

  report << pdim << endl;
  report.setf(ios::scientific);
  for(int i=1;i<=pdim;i++){
    for(int j=1;j<=pdim;j++){
      if(write_ij) report << "(" << i << "," << j << ")";
      if(howtowrite==0) report << mid(Dx[i][j]).right_b() << "  ";
      else writeinterval(report,Dx[i][j],howtowrite);
    }
    report << endl;
  }
  report.flags(oldflags);
}

void saveArray(ostream& report,const double Dx[][MAX_DIM+1],int pdim,
  int /* dummy =0*/, bool write_ij /*= false*/)
{

#if defined(BORLAND)
  long int
 #else
    ios::fmtflags
 #endif
 oldflags=report.flags();


  int i,j;
  report << pdim << endl;
/*  report.setf(ios::fixed);
  report.unsetf(ios::scientific); */

  report.unsetf(ios::fixed);
  report.setf(ios::scientific);

  report.precision(5);
  for(i=1;i<=pdim;i++){
    for(j=1;j<=pdim;j++){
      if(write_ij) report << "(" << i << "," << j << ")";
      report << setw(10) << Dx[i][j] << " ";
    }
    report << endl;
  }
  report.flags(oldflags);
}


void readmatrix(istream& report,double Dx[][MAX_DIM+1])
{
  int i,j;
  char trash[240];
  int pdim;
  report >> pdim;
  report.getline(trash,240); // to eat endl
  for(i=1;i<=pdim;i++){
    for(j=1;j<=pdim;j++)
      report >> Dx[i][j];
    report.getline(trash,240); // to eat endl
  }
}


void readmatrix(istream& report,interval Dx[][MAX_DIM+1])
{
  int i,j;
  char trash[240];
  int pdim;
  report >> pdim;
  report.getline(trash,240); // to eat endl
  for(i=1;i<=pdim;i++){
    for(j=1;j<=pdim;j++)
      report >> Dx[i][j];
    report.getline(trash,240); // to eat endl
  }
}

void readmatrix(const char *repfname,double Dx[][MAX_DIM+1],int /*pdim*/)
{
  ifstream fhandle(repfname);
  readmatrix(fhandle,Dx);
}

void saveArray(const char* report,const interval Dx[][MAX_DIM+1],int pdim,
   int howtowrite /* =1*/,bool write_ij /*= false*/)
// howtowrite == 0 - only the mid-point is written
//                 1 - interval, standard form
//                 2 - interval : center + [-d,d]
{
  ofstream fhandle(report,ios::out);
  saveArray(fhandle,Dx,pdim,howtowrite,write_ij);
  fhandle.close();
}

void saveArray(const char* report,const double Dx[][MAX_DIM+1],int pdim,
  int /*dummy=0 */, bool write_ij /*= false*/)
{
  ofstream fhandle(report,ios::out);
  saveArray(fhandle,Dx,pdim);
  fhandle.close();
}



void wreigenval(ostream& report ,const double eigenval[][3],int num)
{
  for(int i=1;i<=num;i++){
    report << eigenval[i][1];
    if(eigenval[i][2]!=0){
      report << "   +i*" << eigenval[i][2];
    }
    report << endl;
  }
}


void wreigenval(const char* name,const double eigenval[][3],int num)
{
  ofstream report(name);
  wreigenval(report,eigenval,num);
}



void wrdiag2x2(ostream &report,const double a[][MAX_DIM+1])
// writes to <report> ; eigenvalues and eigenvectors of <a>
// dim(<a>)=2
{
  double d=det2x2(a);
  double t=trace2x2(a);
  // characteristic equation:  x^2 - t x + d =0
  double delta=t*t - 4*d;
  if(delta>=0){
    double lambda1,lambda2;
    double wek1[MAX_DIM+1],wek2[MAX_DIM+1];
    if(diag2x2(a,&lambda1,wek1,&lambda2,wek2)){
      report.precision(10);
      report << "lambda1=" << lambda1 << "  eigenvect:  ";
      saveAllCoord(report,wek1,2,0);

      report << "lambda2=" << lambda2 << "  eigenvect: ";
      saveAllCoord(report,wek2,2,0);
    }else{
      report << "problem withe the diagonalization " << endl;
    }
  } else{
    report << "Eigenvalues: " << endl;
    report << t/2 << "+-i*" << sqrt(-delta)/2 ;
  }
  report << endl;
}


void wrdiag2x2(ostream &report,const interval a[][MAX_DIM+1])
// writes to <report> ; eigenvalues and eigenvectors of <a>
// dim(<a>)=2
{
  interval d=a[1][1]*a[2][2] - a[1][2]*a[2][1];
  interval t=a[1][1] + a[2][2];
  // characteristic equation:  x^2 - t x + d =0
  interval delta=t*t - 4*d;
  if(delta.contains(0)){
    report << "cannot diagonalize - delta contains 0 " << endl;
  }
  else{
    if(delta>=0){
      interval lambda1,lambda2;
      interval wek1[3],wek2[3];
      if(diag2x2(a,&lambda1,wek1,&lambda2,wek2)){
        report.precision(10);
        report << "lambda1=" << lambda1 << "  eigenvect:";
        saveAllCoord(report,wek1,2,2);

        report << "lambda2=" << lambda2 << "  eigenvect:";
        saveAllCoord(report,wek2,2,2);
      }else{
        report << "the problem with the diagonalization " << endl;
      }
    } else{
      report << "Eigenvalues: " << endl;
      report << t/2 << "+-i*" << sqrt(-delta)/2 ;
    }
  }
  report << endl;
}



void wrmatrix(const double a[][MAX_DIM+1],int dim,ostream& report, const char* comment)
//  writes matrix with some comments
{
  saveArray(report,a,dim);
  report << endl ;
  report  << comment;
  report << endl << endl << endl;
  report << "---------------------------------"<< endl;
  report.flush();
}


void wrmatrix(const double a[][MAX_DIM+1],int dim,const char* fname,
      const char* comment)
//  writes matrix with some comments
{
  ofstream report(fname,ios::app);
  wrmatrix(a,dim,report,comment);
}



#if defined(BORLAND)
void writetime(ostream &fout)
{
  struct  time t;
  gettime(&t);
  fout << "Time hh:min:sec " << int(t.ti_hour) << ":" << int(t.ti_min)
             << ":" << int(t.ti_sec) << endl;
  //, t.ti_hund);

}
void writedate(ostream &fout)
{
  struct date d;
  getdate(&d);
  fout << "DATE (Y.M.D) " << d.da_year << "." <<  int(d.da_mon) << "." <<
                 int(d.da_day) << endl;
}
#endif

void writedatetime(ostream &fout)
{
  #if defined(BORLAND)
    writedate(fout);
    writetime(fout);
  #else
    time_t dtime;
    time (&dtime);
    fout << ctime(&dtime);
  #endif
}


void readp(ifstream &tr,int dimintraj,double *t,double p[])
// the first number in the line is t (time) and then the coordinates follow
{
  char line[500];
  tr >> *t;
  for(int i=1; i<= dimintraj ; i++)
    tr>> p[i];
  tr.getline(line,sizeof(line)); // to eat '\n' and eventually
                        // further coordinates
}



int pointfromtrajfile(const char traj[],double xs[],int dim,double bias)
    // reads a point from the trajectory file <traj>
    // <traj> has the following structure
    //  1-line - the dimension of the phase space <dimintraj>
    //  other lines
    //   t(time) a_1 a_2 .... a_{dimintraj}
  /// On output
  //    xs - contains coordinates of the first point for which t>=bias
  //    xs is <dim> dimensional, if dimintraj > dim then we
  //       use only dim coordinates, otherwise
  //        xs[i]=0 for  dimintraj <i <=dim
{
  zero(xs,dim);
  int ok=0;
  int dimintraj;
  ifstream tr(traj);
  tr >> dimintraj;
  char trash[240];
  tr.getline(trash,240); // to eat endl;

  double t=bias-1.0;
  double *p1;
  p1=new double[dimintraj+1];
  long pointno=0;
  while ((!tr.eof()) && (t<bias)){
    readp(tr,dimintraj,&t,p1);
    pointno++;
  }
  if(t>=bias){
    ok=1;
    int ldim=dim;
    if(ldim > dimintraj) ldim=dimintraj;
    for(int i=1;i<=ldim;i++)      xs[i]=p1[i];
  }
  delete [] p1;
  return ok;
}


void   writedivtable(ostream &fout,const int divtable[],int dimdiv)
{
  if(dimdiv==0){
//    fout << "no division of the domain" << endl;
  }else{
    fout << "the domain is divided as follows: ";
    for(int i=1;i<=dimdiv;i++)
      fout << divtable[i] << "  " ;
    fout << endl;
  }
  fout.flush();
}

#ifdef KRAK
void   writedivtable(FRAME &fout,const int divtable[],int dimdiv)
{
  if(dimdiv==0){
  //  fout << "no division of the domain" << endl;
  }else{
    fout << "the domain is divided as follows: ";
    for(int i=1;i<=dimdiv;i++)
      fout << divtable[i] << "  " ;
    fout << "\n";
  }
}
#endif

int readMapfromFile(ifstream& fhandle, double points[][3],int *num_of_pts,int
     sizeofpoints)
{
 (*num_of_pts)=0;
  while ((!fhandle.eof()) && ((*num_of_pts) < sizeofpoints)){
    (*num_of_pts)++;
    for(int i=1;i<=2;i++){
      fhandle >> points[*num_of_pts][i];
    }
  }
  (*num_of_pts)-- ;  // because allways 0 is read at the end WHY???
  return 1;
}

int readMapfromFile(const char name[],double points[][3],int *num_of_pts,int sizeofpoints)
{
  (*num_of_pts)=0;
  ifstream fhandle(name);
  if(!fhandle){
    #if defined(WIN95) && defined(KRAK)
    rootFrame << "File not found : " << name << "\n";
    waitBt();
    #else
      cout << "File not found : " << name << "\n";
    #endif
    exit(1);
    return 0;
  }
  return  readMapfromFile(fhandle,points,num_of_pts,sizeofpoints);
}


int writebin(const char* filename,const double p[],int dim)
// writes in binary mode:
//  dim and the vector p[1],\dots,p[dim]
{
  FILE* fhandle;
  fhandle=fopen(filename,"wb");
  int ret=(fhandle!=NULL);
  if(ret){
    fwrite(&dim,sizeof(int),1,fhandle);
    fwrite(p+1,sizeof(double),dim,fhandle);
    fclose(fhandle);
  }else{
    cerr << "cannot open " << filename << endl;
     #if defined(KRAK) && defined(WIN95)
                            waitBt();
     #endif
    exit(EXIT_FAILURE);
  }
  return ret;
}

int readbin(const char* filename,double p[],int dim,int *diminfile)
// reads in binary mode:
//  dim and the vector p[1],\dots,p[dim]
//  if the dimension in <filename> is smaller, then
//    p is padded with zeros, otherwise it is truncated
{
  FILE* fhandle;
  fhandle=fopen(filename,"rb");
  int ret=(fhandle!=NULL);
  if(ret){
    fread(diminfile,sizeof(int),1,fhandle);
    double* lp;
    lp=new double[*diminfile+1];
    fread(lp+1,sizeof(double),dim,fhandle);
    fclose(fhandle);

    int i=1;
    while(i<=dim){
      if(i<=*diminfile){
        p[i]=lp[i];
      } else p[i]=0;
      i++;
    }
    delete [] lp;
  }else{
    cerr << "cannot open " << filename << endl;
     #if defined(KRAK) && defined(WIN95)
                            waitBt();
     #endif
    exit(EXIT_FAILURE);
  }
  return ret;
}


void writeCompInfo(ostream &fout)
{
#if defined(ZGLVER)
  fout << "programm version " << ZGLVER << endl;
#endif
#if defined(LINUX)
  fout << "Linux  ";
#endif
#if defined(WIN95)
  fout << "Windows  " ;
#endif
#if defined(SUN_OS)
  fout << "Sun or SGI   ";
#endif
#if defined(BORLAND)
  fout << "Borland C++  ";
#endif
#if defined(GCC_MS_WIN)
  fout << "gcc  ";
#endif
  fout << endl;
}



void saveinitialcond(ostream &report,const interval ixb[],const interval r0[],int dim)
{
    report << " x0 + r " << endl;
    report << "x0: " << endl;
    saveAllCoord(report,ixb,dim,0);
    report << "r0:" << endl;
    saveAllCoord(report,r0,dim,1); // 1 - standard interval form
}

void write_compar_quotients(ostream & report,const interval fx[],
       const interval rcomp[],int dim)
{
  int numbigratios=0;
  double largestratio=0;

  for(int i=1;i<=dim;i++){
    if(diameter(rcomp[i]).right_b() > 0){
      double r=diameter(fx[i]).right_b()/diameter(rcomp[i]).right_b();
      if(r > 1) numbigratios++;
      if(r > largestratio) largestratio=r;
      report << r << "  ";
    }
  }
  report << endl;
  report << "Number of ratios > 1 is = " << numbigratios << endl;
  report << endl;
}


void write_empty_intersection(ostream &report,const interval x[],const interval fx[],
      int dim,const char *nx,const char *nfx)
{
  report << nx << " \\cap " << nfx << " = empty set " << endl;
  report << "listing of bad coordinates: " << endl;

  for(int i=1;i<=dim;i++){
    interval cpi;
    if(!commonpart(x[i],fx[i],&cpi)){
      report << "i=" << i << " " << nx << "[i]=" << x[i] <<
 "  " << nfx <<"[i]=" << fx[i] << endl;
    }
  }
}


int save_points_bin(const char fbin[],int dim,doublematrix1 p[],int first,int last)
// the structure of the file:
// (int)version=1,(int)dim,(int)number of points,p[first],p[first+1],...,p[last]
// p[i] - is saved as sequence of dim "doubles"
// p[i][1],...,p[i][dim] - the index in p[i] starts from 1
//  version=1 - indicates that we save doubles
//
{
  int pointnumber=last-first +1;
  FILE* f;
  f=fopen(fbin,"wb");
  int ret=(f!=NULL);
  if(ret){
    int d=1;
    fwrite(&d,sizeof(d),1,f);
    fwrite(&dim,sizeof(dim),1,f);
    fwrite(&pointnumber,sizeof(pointnumber),1,f);
    for(int i=first;i<=last;i++){
      fwrite(p[i]+1,sizeof(double),dim,f);
    }
    fclose(f);
  }else{
    cerr << "cannot open " << fbin << endl;
     #if defined(KRAK) && defined(WIN95)
                            waitBt();
     #endif
    exit(EXIT_FAILURE);
  }
  return ret;
}


int read_points_bin(const char fbin[],int *dim,doublematrix1 **p,int *ptnumber)
// the inverse function to <save_points_bin>
//  on output:
//    *dim - the dimension of points in the file
//    *ptnumber - the number of points
//    (*p)[i] , for i=0,..,*ptnumber-1 - are the <dim>-dimensional points
//                 with the coordinate index starting from 1
{
  FILE* f;
  f=fopen(fbin,"rb");
  int ret=(f!=NULL);
  if(ret){
    int d;
    d=fread(&d,sizeof(d),1,f);
    if(d!=1){
      ret=0;
      cerr << "read_points_bin - apprently incorrect version of the file " << endl;
      #if defined(WIN95) && defined(KRAK)
        waitBt();
      #endif
      exit(EXIT_FAILURE);
    }
    fread(dim,sizeof(*dim),1,f);
    if(*dim <=MAX_DIM){
      fread(ptnumber,sizeof(*ptnumber),1,f);
      *p=new doublematrix1[*ptnumber];
      for(int i=0;i<*ptnumber;i++){
        fread((*p)[i]+1,sizeof(double),*dim,f);
      }
    }else{
      ret=0;
      cerr << "the dimension in " << fbin << "  is too large " << *dim << endl;
      #if defined(WIN95) && defined(KRAK)
        waitBt();
      #endif
      exit(EXIT_FAILURE);
    }
    fclose(f);
  }else{
    cerr << "cannot open " << fbin << endl;
     #if defined(KRAK) && defined(WIN95)
                            waitBt();
     #endif
    exit(EXIT_FAILURE);
  }
  return ret;
}


int copyfile(const char source[],const char dest[])
// I don't know why this function does not work under windows (Borland)
// but it works fine under Linux
{

  int len=strlen(source)+strlen(dest)+20;;
  char *command=new char[len];
  if(fileexists(dest)){
    #if defined(WIN95)
      strcpy(command,"command /c del ");
    #else
      strcpy(command,"rm ");
    #endif
    strcat(command,dest);
    if(system(command)==-1){
      cerr <<  "cannot delete file " << dest << endl;
       #if defined(KRAK) && defined(WIN95)
                              waitBt();
     #endif
      exit(EXIT_FAILURE);
    }
  }
  #if defined(WIN95)
    strcpy(command,"command /c copy ");
  #else
    strcpy(command,"cp ");
  #endif
  strcat(command,source);
  strcat(command,"  ");
  strcat(command,dest);
  if(system(command)==-1){
    cerr <<  "copy file " << source << " to  "<< dest << endl;
     #if defined(KRAK) && defined(WIN95)
                            waitBt();
     #endif
    exit(EXIT_FAILURE);
  }
  delete [] command;
  return 1;
}


void writeratios(ostream &report,const interval n[],const interval d[],int pdim)
// function writes to file the sequence of ratios diameter(n[i])/diameter(d[i] )
{
  for(int i=1;i<=pdim;i++)
  {
    if( diameter(d[i]).right_b()>0  ){
      report <<   diameter(n[i]).right_b()/diameter(d[i]).right_b() << "   ";
    } else{
      report << " infty ";
    }
  }
  report << endl;
}


interval enterIntervalAsRatio(const char* varname )
//  we return the intrval defined by p/q, where p,q are integers
//   provided by the user
{
  long int nominator=0;
  long int denominator=1;
  cout << "nominator of " << varname << "=";
  cin >> nominator;
  #if defined(WIN95) && defined(KRAK)
  cout << endl;
  #endif
  if(nominator!=0){
    cout << "denominator of " << varname << "=";
    cin >> denominator;
    #if defined(WIN95) && defined(KRAK)
    cout << endl;
    #endif
  }
  if(denominator==0){
    cerr << "ERROR - zero in the denominator" << endl;
    #if defined(WIN95) && defined(KRAK)
    cout << endl;
    #endif
    exit(EXIT_FAILURE);
  }
  return interval(nominator)/interval(denominator);
}
